#define USE_CONSOLE

#include <string>
#include <allegro.h>
#include <assert.h>
#include <string.h>
#include <time.h>

// CONSTANTS

const int SCREEN_WIDTH  = 800;
const int SCREEN_HEIGHT = 600;
const int ROAD_WIDTH = 50;
const int ROAD_Y = 350;
const int STATUSBAR_HEIGHT = 16;

const int PERSON_SPEED                 = 2;
const int TABLOID_MAX_PICKUP_DISTANCE  = 40;
const int TABLOID_OFFSET_CARRY_X       = 0;
const int TABLOID_OFFSET_CARRY_Y       = 31;
const int TABLOID_OFFSET_PUTDOWN_X     = 0;
const int TABLOID_OFFSET_PUTDOWN_Y     = 42;

const int BUILDING_MAX_ENTER_DISTANCE  = 100;

const int LOOPS_PER_FRAME  = 20;
const int MAX_FRAMES       = 10;

const int FIRE_LOOPS_MIN      = 600;
const int FIRE_LOOPS_MAX      = 800;

const int DELIVERY_LOOPS_MIN        = 400;  
const int DELIVERY_LOOPS_MAX        = 700;  
const int DELIVERY_NUM_TABLOIDS_MIN = 2;    
const int DELIVERY_NUM_TABLOIDS_MAX = 7;    

const int STAT_LOOPS_PER_CHANGE  = 100;
const int STAT_INIT_POPULARITY   = 50;
const int STAT_INIT_HEALTH       = 50;
const int STAT_INIT_FUN          = 50;
const int STAT_DELTA_POPULARITY  = 0;
const int STAT_DELTA_HEALTH      = 0;
const int STAT_DELTA_FUN         = 0;
const int STAT_CLUB_POPULARITY   = 0;
const int STAT_CLUB_HEALTH       = -1;
const int STAT_CLUB_FUN          = 4;
const int STAT_BARBER_POPULARITY = 0;
const int STAT_BARBER_HEALTH     = 0;
const int STAT_BARBER_FUN        = 0;
const int STAT_REHAB_POPULARITY  = 0;
const int STAT_REHAB_HEALTH      = 4;
const int STAT_REHAB_FUN         = -1;
const int STAT_DELTA_POPULARITY_NPC = -5;

const int NPC_ACTION_LOOPS_MIN   = 300;
const int NPC_ACTION_LOOPS_MAX   = 600;

// STRUCTS

struct GameObj
{
   int x, y;                  // position
   int w, h, half_w, half_h;  // size
   BITMAP *bmp;
};

struct Building : GameObj
{
   void StatChangePersonInMe( int &pop, int &health, int &fun )
   {
      pop += statPopularityDelta;
      health += statHealthDelta;
      fun += statFunDelta;
   }

   int statPopularityDelta;
   int statHealthDelta;
   int statFunDelta;

   BITMAP *bmps[2];
   //Person *containedPerson;
   int prevPersonX, prevPersonY;
};

struct Tabloid : GameObj
{
   int fireState;
   int loopsUntilBurnedOut;
   bool isCarried;
};

struct Person : GameObj
{
   int carryState;
   Building* buildingCurrent;
   Building* buildingHome;
   int loopsTilNextAction;
   int state;
   Tabloid *stateTabloid;
   Building *stateBuilding;
};

struct Animatable : GameObj
{
   int numFrames;
   int curFrame;
   int loopsPerFrame;
   int loopCount;
   BITMAP *frames[MAX_FRAMES];
   bool visible;
};

// ENUMS

enum {britney, paris, /*kfed, sean, jayden,*/ numPeople};
enum {brithouse, club, barber, rehab, parishouse, lohanhouse, justinhouse, numBuildings};
enum {numTabloids = 1000};

enum {carry_none, carry_tabloid};
//enum {in_building_no, in_building_yes};
enum {fire_off, fire_on};
enum {personstate_default, personstate_gotobuilding, personstate_tabloid_get, personstate_tabloid_pickup};

// GLOBAL VARS

BITMAP *gBackBuf;
BITMAP *gBmpTabloidHighlightOn, *gBmpTabloidHighlightOff;
BITMAP *gBmpHouseOff, *gBmpHouseOn;
BITMAP *gBmpFire;

bool gWasSpacePressedLastLoop;

Person      people[numPeople];
Building    buildings[numBuildings];
Tabloid     tabloids[numTabloids];

Tabloid  *gNearestTabloid;
Building *gNearestBuilding;

Animatable gFirePatch;

int   gStatCurLoop;
int   gPopularity;
int   gHealth;
int   gFun;
int   gLoopsUntilDelivery;

// FUNCTIONS

int Random( int min, int max )
{
   assert( min < max );
   if( min < max )
   {
      //srand( time( NULL ) );
      return( ( rand() % ( max - min + 1 ) ) + min );
   }
   else
   {
      return 0;
   }
}

bool IsTabloidDeleted( Tabloid &tab )
{
   return( tab.loopsUntilBurnedOut <= 0 );
}

bool IsNearestBuilding( Building *bldg )
{
   if( bldg == gNearestBuilding )
   {
      return true;
   }
   else
   {
      return false;
   }
}

bool IsNearestTabloid( GameObj *tab )
{
   if( tab == gNearestTabloid )
   {
      return true;
   }
   else
   {
      return false;
   }
}

void DrawPeople()
{
   for( int i = 0; i < numPeople; i++ )
   {
      if( i == britney || people[i].state != personstate_default )
      {
         draw_sprite( gBackBuf, people[i].bmp, people[i].x, people[i].y );
      }
   }
}

void DrawBuildings()
{
   for( int i = 0; i < numBuildings; i++ )
   {
      if( IsNearestBuilding( &buildings[i] ) )
      {
         buildings[i].bmp = buildings[i].bmps[1];
      }
      else
      {
         buildings[i].bmp = buildings[i].bmps[0];
      }

      draw_sprite( gBackBuf, buildings[i].bmp, buildings[i].x, buildings[i].y );
   }
}

void DrawTabloids()
{
   // tabloids
   for( int i = 0; i < numTabloids; i++ )
   {
      // don't draw tabloid if it's burned out (i.e. "deleted")
      if( !IsTabloidDeleted( tabloids[i] ) )
      {
         if( IsNearestTabloid( &tabloids[i] ) && tabloids[i].fireState == fire_off )
         {
            tabloids[i].bmp = gBmpTabloidHighlightOn;
         }
         else
         {
            tabloids[i].bmp = gBmpTabloidHighlightOff;
         }

         draw_sprite( gBackBuf, tabloids[i].bmp, tabloids[i].x, tabloids[i].y );
      }
   }

   // fires
   for( int i = 0; i < numTabloids; i++ )
   {
      if( tabloids[i].fireState == fire_on )
      {
         draw_sprite( gBackBuf, gBmpFire, tabloids[i].x, tabloids[i].y );
      }
   }
}

void DrawCarriedTabloid()
{
   for( int i = 0; i < numPeople; i++ )
   {
      if( people[i].carryState == carry_tabloid )
      {
         if( i == britney )
         {
            assert( gNearestTabloid );
            if( gNearestTabloid )
            {
               draw_sprite( gBackBuf, gNearestTabloid->bmp, gNearestTabloid->x, gNearestTabloid->y );
            }
         }
         else  // npc
         {
            assert( people[i].stateTabloid );
            if( people[i].stateTabloid )
            {
               draw_sprite( gBackBuf, people[i].stateTabloid->bmp, people[i].stateTabloid->x, people[i].stateTabloid->y );
            }
         }
      }
   }

   /*
   if( people[britney].carryState == carry_tabloid )
   {
      assert( gNearestTabloid );
      if( gNearestTabloid )
      {
         draw_sprite( gBackBuf, gNearestTabloid->bmp, gNearestTabloid->x, gNearestTabloid->y );
      }
   }
   */
}

void DrawFirePatch()
{
   if( gFirePatch.visible )
   {
      // update animation
      gFirePatch.loopCount++;
      if( gFirePatch.loopCount >= gFirePatch.loopsPerFrame )
      {
         gFirePatch.loopCount = 0;
         gFirePatch.curFrame++;

         if( gFirePatch.curFrame >= gFirePatch.numFrames )
         {
            gFirePatch.curFrame = 0;
         }

         gFirePatch.bmp = gFirePatch.frames[gFirePatch.curFrame];
      }

      // draw
      draw_sprite( gBackBuf, gFirePatch.bmp, gFirePatch.x, gFirePatch.y );
   }
}

void DrawBackground()
{
   // upper row of lots
   rectfill( gBackBuf, 0, 0, 200, ROAD_Y, makecol( 0, 128, 0 ) );
   rectfill( gBackBuf, 200, 0, 600, ROAD_Y, makecol( 0, 192, 0 ) );
   rectfill( gBackBuf, 600, 0, SCREEN_W, ROAD_Y, makecol( 0, 128, 64 ) );

   // lower row of lots
   rectfill( gBackBuf, 0, ROAD_Y, 200, SCREEN_H, makecol( 0, 160, 32 ) );
   rectfill( gBackBuf, 200, ROAD_Y, 400, SCREEN_H, makecol( 0, 192, 64 ) );
   rectfill( gBackBuf, 400, ROAD_Y, 600, SCREEN_H, makecol( 0, 128, 32 ) );
   rectfill( gBackBuf, 600, ROAD_Y, SCREEN_W, SCREEN_H, makecol( 0, 192, 32 ) );

   // road
   rectfill( gBackBuf, 0, ROAD_Y - ROAD_WIDTH, SCREEN_W, ROAD_Y + ROAD_WIDTH, makecol( 64, 64, 64 ) );

   // fire patch
   DrawFirePatch();
}

void DrawText()
{
   rectfill( gBackBuf, 0, 0, SCREEN_W, STATUSBAR_HEIGHT, makecol( 255, 255, 255 ) );
   
   char tempStr[128];
   sprintf( tempStr, "POPULARITY: %d %%", gPopularity );
   textout_ex( gBackBuf, font, tempStr, 100, 4, makecol( 255, 0, 0 ), -1 );

   sprintf( tempStr, "Health: %d %%", gHealth );
   textout_ex( gBackBuf, font, tempStr, 300, 4, makecol( 0, 128, 0 ), -1 );

   sprintf( tempStr, "Fun: %d %%", gFun );
   textout_ex( gBackBuf, font, tempStr, 500, 4, makecol( 0, 0, 255 ), -1 );
}

void Draw()
{
   //clear_to_color( gBackBuf, makecol( 0, 128, 0 ) );
   //textout_centre_ex( bmp, font, "Hello, world!", SCREEN_W/2, SCREEN_H/2, makecol(0,0,0), -1 );

   DrawBackground();
   DrawBuildings();
   DrawTabloids();
   DrawPeople();
   DrawCarriedTabloid();
   DrawText();

   vsync();
	acquire_screen();
	blit( gBackBuf, screen, 0, 0, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT );
	release_screen();
}

bool IsOutOfBounds( GameObj &obj )
{
   if( obj.x < 0 || obj.x + obj.w > SCREEN_W - 1 || obj.y < STATUSBAR_HEIGHT + 1 || obj.y + obj.h > SCREEN_H - 1 )
   {
      return true;
   }
   else
   {
      return false;
   }
}

bool IsObjectOnRoad( GameObj &obj )
{
   return( obj.y + obj.h > ROAD_Y - ROAD_WIDTH && obj.y < ROAD_Y + ROAD_WIDTH );
}

bool AreObjectsOverlapping( GameObj &obj1, GameObj &obj2 )
{
   if( ( obj1.x + obj1.w > obj2.x ) && ( obj1.x < obj2.x + obj2.w ) &&
       ( obj1.y + obj1.h > obj2.y ) && ( obj1.y < obj2.y + obj2.h ) )
   {
      return true;
   }
   else
   {
      return false;
   }
}

// only consider lower half of adult bitmap in object collision
// (works well for dropping tabloids)
bool IsAdultOverlapping( GameObj &adult, GameObj &obj2 )
{
   if( ( adult.x + adult.w > obj2.x ) && ( adult.x < obj2.x + obj2.w ) &&
       ( adult.y + adult.h > obj2.y ) && ( adult.y + adult.half_h < obj2.y + obj2.h ) )
   {
      return true;
   }
   else
   {
      return false;
   }
}

// only consider lower half of adult bitmap in object collision
bool IsAdultTouching( GameObj &adult, GameObj &obj2 )
{
   if( ( adult.x + adult.w > obj2.x - PERSON_SPEED * 2 ) &&
       ( adult.x < obj2.x + obj2.w + PERSON_SPEED * 2 ) &&
       ( adult.y + adult.h > obj2.y - PERSON_SPEED * 2 ) &&
       ( adult.y + adult.half_h < obj2.y + obj2.h + PERSON_SPEED * 3 ) )
   {
      return true;
   }
   else
   {
      return false;
   }
}

void FirePatchSetVisible( bool isVisible )
{
   if( isVisible )
   {
      gFirePatch.visible = true;
   }
   else
   {
      gFirePatch.visible = false;
      gFirePatch.curFrame = 0;
      gFirePatch.loopCount = 0;
   }
}

// Sets the global "nearest building" ptr to the building nearest to
// britney if at least one is within maxDistance, otherwise returns null.
int SetNearestBuilding()
{
   gNearestBuilding = NULL;
   int currentShortestDistanceSquared = BUILDING_MAX_ENTER_DISTANCE * BUILDING_MAX_ENTER_DISTANCE;
   
   for( int i = 0; i < numBuildings; i++ )
   {
      int x = abs( ( people[britney].x + people[britney].half_w ) - ( buildings[i].x + buildings[i].half_w ) );
      int y = abs( ( people[britney].y + people[britney].half_h ) - ( buildings[i].y + buildings[i].half_h ) );

      int z_squared = x*x + y*y;

      if( z_squared < currentShortestDistanceSquared )
      {
         gNearestBuilding = &buildings[i];
         currentShortestDistanceSquared = z_squared;
      }
   }

   return currentShortestDistanceSquared;
}

Tabloid *FindNearestTabloidForNPC( Person &npc )
{
   int currentShortestDistanceSquared = SCREEN_W * SCREEN_W;

   Tabloid *nearestTabloid = NULL;

   for( int i = 0; i < numTabloids; i++ )
   {
      if( !IsTabloidDeleted( tabloids[i] ) && &tabloids[i] != gNearestTabloid )
      {
         int x = abs( ( npc.x + npc.half_w ) - ( tabloids[i].x + tabloids[i].half_w ) );
         int y = abs( ( npc.y + npc.half_h ) - ( tabloids[i].y + tabloids[i].half_h ) );

         int z_squared = x*x + y*y;

         if( z_squared < currentShortestDistanceSquared )
         {
            nearestTabloid = &tabloids[i];
            currentShortestDistanceSquared = z_squared;
         }
      }
   }
   
   return nearestTabloid;
}

// Sets the global "nearest tabloid" ptr to the tabloid nearest to
// britney if at least one is within maxDistance, otherwise returns null.
int SetNearestTabloid()
{
   int currentShortestDistanceSquared = TABLOID_MAX_PICKUP_DISTANCE * TABLOID_MAX_PICKUP_DISTANCE;

   // maintain the tabloid she's carrying as the "nearest" one
   if( people[britney].carryState == carry_tabloid )
   {
      return currentShortestDistanceSquared;
   }

   gNearestTabloid = NULL;
   
   for( int i = 0; i < numTabloids; i++ )
   {
      if( !IsTabloidDeleted( tabloids[i] ) && !tabloids[i].isCarried )
      {
         int x = abs( ( people[britney].x + people[britney].half_w ) - ( tabloids[i].x + tabloids[i].half_w ) );
         int y = abs( ( people[britney].y + people[britney].half_h ) - ( tabloids[i].y + tabloids[i].half_h ) );

         int z_squared = x*x + y*y;

         if( z_squared < currentShortestDistanceSquared )
         {
            gNearestTabloid = &tabloids[i];
            currentShortestDistanceSquared = z_squared;
         }
      }
   }
   
   return currentShortestDistanceSquared;
}

// move carried tabloid
void NPCTabloidUpdatePosition( Person &npc )
{
   if( npc.carryState == carry_tabloid )
   {
      //assert( npc.stateTabloid );
      if( npc.stateTabloid )
      {
         // position the tabloid so it looks like npc's carrying it
         npc.stateTabloid->x = npc.x + TABLOID_OFFSET_CARRY_X;
         npc.stateTabloid->y = npc.y + TABLOID_OFFSET_CARRY_Y;
      }
   }
}

// move carried tabloid
void TabloidUpdatePosition()
{
   if( people[britney].carryState == carry_tabloid )
   {
      assert( gNearestTabloid );
      if( gNearestTabloid )
      {
         // position the tabloid so it looks like britney's carrying it
         gNearestTabloid->x = people[britney].x + TABLOID_OFFSET_CARRY_X;
         gNearestTabloid->y = people[britney].y + TABLOID_OFFSET_CARRY_Y;
      }
   }
}

/*
Person *GetAdultInBuilding( Building &building )
{
   return( building.containedPerson );
}
*/

void NPCTabloidPickUp( Person &npc )
{
   // pick up
   if( npc.carryState == carry_none && npc.stateTabloid &&
       npc.stateTabloid->fireState == fire_off && !npc.stateTabloid->isCarried )
   {
      npc.carryState = carry_tabloid;
      npc.stateTabloid->isCarried = true;
      NPCTabloidUpdatePosition( npc );
   }
}

void TabloidPickUpPutDown()
{
   // can't pick up or put down if inside a building
   if( gNearestBuilding && people[britney].buildingCurrent )
   {
      return;
   }

   // pick up
   if( people[britney].carryState == carry_none && gNearestTabloid &&
       gNearestTabloid->fireState == fire_off && !gNearestTabloid->isCarried )
   {
      people[britney].carryState = carry_tabloid;
      gNearestTabloid->isCarried = true;
      TabloidUpdatePosition();

      FirePatchSetVisible( true );
   }
   // put down
   else if( people[britney].carryState == carry_tabloid && gNearestTabloid->isCarried )
   {
      people[britney].carryState = carry_none;
      gNearestTabloid->isCarried = false;

      // position the tabloid so it looks like britney just put it down
      gNearestTabloid->x = people[britney].x + TABLOID_OFFSET_PUTDOWN_X;
      gNearestTabloid->y = people[britney].y + TABLOID_OFFSET_PUTDOWN_Y;

      if( AreObjectsOverlapping( *gNearestTabloid, gFirePatch ) )
      {
         gNearestTabloid->fireState = fire_on;
      }

      SetNearestTabloid();

      FirePatchSetVisible( false );
   }
}

void DeleteTabloid( Tabloid &tabloid )
{
   // "delete" tabloid & fire (if "loops" var is zero then won't be drawn)
   tabloid.fireState = fire_off;
   tabloid.loopsUntilBurnedOut = 0;
   tabloid.isCarried = false;
}

void UpdateFires()
{
   for( int i = 0; i < numTabloids; i++ )
   {
      if( tabloids[i].fireState == fire_on )
      {
         tabloids[i].loopsUntilBurnedOut--;
         if( IsTabloidDeleted( tabloids[i] ) )
         {
            // "delete" tabloid & fire (if "loops" var is zero then won't be drawn)
            /*
            tabloids[i].fireState = fire_off;
            tabloids[i].loopsUntilBurnedOut = 0;
            */
            DeleteTabloid( tabloids[i] );
         }
      }
   }
}

void DoDelivery()
{
   int remainingTabloids = Random( DELIVERY_NUM_TABLOIDS_MIN, DELIVERY_NUM_TABLOIDS_MAX );
   // loop through all tabloids to find a "deleted" one we can re-enable
   for( int i = 0; i < numTabloids; i++ )
   {
      if( IsTabloidDeleted( tabloids[i] ) )
      {
         tabloids[i].x = Random( 0, SCREEN_W - tabloids[i].w );
         tabloids[i].y = Random( STATUSBAR_HEIGHT, SCREEN_H - tabloids[i].h );

         // make sure not intersecting a building or the fire patch or road
         for( int j = 0; j < numBuildings; j++ )
         {
            // pick random displacement direction
            int dir = Random( 1, 4 );
            int whileCount = 0;
            if( IsObjectOnRoad( tabloids[i] ) )
            {
               dir = Random( 3, 4 );
            }
            while( AreObjectsOverlapping( tabloids[i], buildings[j] ) ||
                   AreObjectsOverlapping( tabloids[i], gFirePatch ) ||
                   IsObjectOnRoad( tabloids[i] ) )
            {
               switch( dir )
               {
                  case 1:  tabloids[i].x -= 10; break;
                  case 2:  tabloids[i].x += 10; break;
                  case 3:  tabloids[i].y -= 10; break;
                  case 4:  tabloids[i].y += 10; break;
                  default: assert( 0 == 1 ); tabloids[i].y += 10;
               }

               whileCount++;
               if( whileCount >= 100 )
               {
                  // failsafe, should never hit here
                  assert( whileCount < 100 );
                  break;
               }
            }
         }

         tabloids[i].fireState = fire_off;
         tabloids[i].bmp = gBmpTabloidHighlightOff;
         tabloids[i].loopsUntilBurnedOut = Random( FIRE_LOOPS_MIN, FIRE_LOOPS_MAX );

         remainingTabloids--;
         if( remainingTabloids <= 0 )
         {
            break;
         }
      }
   }
}

void UpdateDelivery()
{
   gLoopsUntilDelivery--;
   if( gLoopsUntilDelivery <= 0 )
   {
      DoDelivery();
      gLoopsUntilDelivery = Random( DELIVERY_LOOPS_MIN, DELIVERY_LOOPS_MAX );
   }
}

void UpdateStats()
{
   gStatCurLoop++;

   if( gStatCurLoop >= STAT_LOOPS_PER_CHANGE )
   {
      // constant baseline stat changes
      gPopularity += STAT_DELTA_POPULARITY;
      gHealth     += STAT_DELTA_HEALTH;
      gFun        += STAT_DELTA_FUN;

      // stat change due to being inside a particular building
      //if( gNearestBuilding && GetAdultInBuilding( *gNearestBuilding ) )
      if( gNearestBuilding && people[britney].buildingCurrent )
      {
         gNearestBuilding->StatChangePersonInMe( gPopularity, gHealth, gFun );
      }

      // cap stats between 0 and 100
      if( gPopularity < 0 )
      {
         gPopularity = 0;
      }
      else if( gPopularity > 100 )
      {
         gPopularity = 100;
      }
      
      if( gHealth < 0 )
      {
         gHealth = 0;
      }
      else if( gHealth > 100 )
      {
         gHealth = 100;
      }
      
      if( gFun < 0 )
      {
         gFun = 0;
      }
      else if( gFun > 100 )
      {
         gFun = 100;
      }

      gStatCurLoop = 0;
   }
}

void UpdateNearestObjects()
{
   int distTabloid   = SetNearestTabloid();    // find the nearest tabloid each loop (for picking up and turning on highlight)
   int distBuilding  = SetNearestBuilding();

   // decide which objects are ok to highlight, e.g. only tabloid, only building, tabloid & building, etc.
   
   // if britney is touching the edge of the building then highlight it,
   // even if a tabloid is mathematically closer (unless she's holding that tabloid)
   if( gNearestBuilding && IsAdultTouching( people[britney], *gNearestBuilding ) )
   {
      if( people[britney].carryState == carry_none )
      {
         gNearestTabloid = NULL;
      }
   }
   else
   {
      gNearestBuilding = NULL;
   }
}

void UpdateNPCs()
{
   for( int i = 0; i < numPeople; i++ )
   {
      NPCTabloidUpdatePosition( people[i] );

      // start new action?
      if( i != britney && people[i].state == personstate_default )
      {
         people[i].loopsTilNextAction--;

         if( people[i].loopsTilNextAction <= 0 )
         {
            // for now just have them get a tabloid
            people[i].state = personstate_tabloid_get;
            people[i].stateTabloid = FindNearestTabloidForNPC( people[i] );

            people[i].loopsTilNextAction = Random( NPC_ACTION_LOOPS_MIN, NPC_ACTION_LOOPS_MAX );
         }
      }
   }
}

void Update()
{
   UpdateFires();
   UpdateDelivery();
   UpdateStats();
   UpdateNearestObjects();
   UpdateNPCs();
}

// returns true if the building was entered or exited, false otherwise
bool BuildingEnterExit()
{
   // enter
   //if( gNearestBuilding && !GetAdultInBuilding( *gNearestBuilding ) )
   if( gNearestBuilding && people[britney].buildingCurrent == NULL )
   {
      //gNearestBuilding->containedPerson = &people[britney];
      people[britney].buildingCurrent = gNearestBuilding;

      gNearestBuilding->prevPersonX = people[britney].x;
      gNearestBuilding->prevPersonY = people[britney].y;
      people[britney].x = gNearestBuilding->x + gNearestBuilding->half_w - people[britney].half_w;
      people[britney].y = gNearestBuilding->y + gNearestBuilding->half_h - people[britney].half_h;

      // update position of carried tabloid
      TabloidUpdatePosition();
      
      // turn fire patch off
      if( people[britney].carryState == carry_tabloid )
      {
         FirePatchSetVisible( false );
      }

      return true;
   }
   // exit
   else if( gNearestBuilding && people[britney].buildingCurrent )
   {
      //gNearestBuilding->containedPerson = NULL;
      people[britney].buildingCurrent = NULL;

      people[britney].x = gNearestBuilding->prevPersonX;
      people[britney].y = gNearestBuilding->prevPersonY;
      gNearestBuilding->prevPersonX = 0;
      gNearestBuilding->prevPersonY = 0;

      // update position of carried tabloid
      TabloidUpdatePosition();

      // turn fire patch back on
      if( people[britney].carryState == carry_tabloid )
      {
         FirePatchSetVisible( true );
      }

      return true;
   }

   return false;
}

void SpacePressed()
{
   if( !BuildingEnterExit() )
   {
      TabloidPickUpPutDown();
   }
}

//void NPCMoveTowardXY( Person &npc, int x, int y )
void NPCMoveTowardObject( Person &npc, GameObj &obj )
{
   int deltaX = ( obj.x + obj.half_w ) - ( npc.x + npc.half_w );
   int deltaY = ( obj.y + obj.half_h ) - ( npc.y + npc.half_h );
   if( abs( deltaX ) > PERSON_SPEED )
   {
      if( deltaX < 0 )
      {
         npc.x -= PERSON_SPEED;
      }
      else
      {
         npc.x += PERSON_SPEED;
      }      
   }
   if( abs( deltaY ) > PERSON_SPEED )
   {
      if( deltaY < 0 )
      {
         npc.y -= PERSON_SPEED;
      }
      else
      {
         npc.y += PERSON_SPEED;
      }
   }
}

void NPCTabloidGet( Person &npc )
{
   // verify the tabloid is still a valid target
   if( IsTabloidDeleted( *(npc.stateTabloid) ) || npc.stateTabloid->fireState == fire_on || npc.stateTabloid->isCarried )
   {
      // state change: go back home
      npc.carryState = carry_none;
      npc.stateTabloid = NULL;
      npc.state = personstate_gotobuilding;
      npc.stateBuilding = npc.buildingHome;
      return;
   }

   // are we there yet?
   if( IsAdultOverlapping( npc, *(npc.stateTabloid) ) )
   {
      // pick up the tabloid
      NPCTabloidPickUp( npc );

      // state change: go home with tabloid
      npc.state = personstate_gotobuilding;
      npc.stateBuilding = npc.buildingHome;
      return;
   }

   // move toward tabloid
   //NPCMoveTowardXY( npc, npc.x, npc.y );
   //NPCMoveTowardObject( npc, npc.stateTabloid->x, npc.stateTabloid->y );
   NPCMoveTowardObject( npc, *npc.stateTabloid );
}

void NPCGoToBuilding( Person &npc )
{
   // are we there yet?
   if( IsAdultOverlapping( npc, *(npc.stateBuilding) ) )
   {
      if( npc.carryState = carry_tabloid && npc.stateTabloid && npc.stateTabloid->isCarried )
      {
         // "delete" the tabloid
         DeleteTabloid( *npc.stateTabloid );
         npc.carryState = carry_none;
         npc.stateTabloid = NULL;
         gPopularity += STAT_DELTA_POPULARITY_NPC;
         if( gPopularity < 0 )
         {
            gPopularity = 0;
         }
      }

      // state change: back to default
      npc.state = personstate_default;
      npc.stateBuilding = NULL;
      return;
   }

   // if we're not there yet then move toward our destination
   //NPCMoveTowardXY( npc, npc.stateBuilding->x, npc.stateBuilding->y );
   NPCMoveTowardObject( npc, *npc.stateBuilding );
}

void InputNPCs()
{
   for( int i = 0; i < numPeople; i++ )
   {
      if( i != britney && people[i].state != personstate_default )
      {
         switch( people[i].state )
         {
            case personstate_tabloid_get:
               NPCTabloidGet( people[i] ); 
               break;

            case personstate_gotobuilding:
               NPCGoToBuilding( people[i] ); 
               break;
         }
      }
   }
}

int Input()
{
   // handle events
	if( keypressed() )
   {
      // player movement
      if( key[KEY_UP] || key[KEY_DOWN] || key[KEY_LEFT] || key[KEY_RIGHT] )
      {
         // move vertically
         int oldy = people[britney].y; // save original position
         if( key[KEY_UP] )    people[britney].y -= PERSON_SPEED;
         if( key[KEY_DOWN] )  people[britney].y += PERSON_SPEED;
         
         // collision detection
         for( int i = 0; i < numBuildings; i++ )
         {
            if( IsAdultOverlapping( people[britney], buildings[i] ) || IsOutOfBounds( people[britney] ) )
            {
               // restore original position
               people[britney].y = oldy;
               oldy = people[britney].y;
            }
         }

         // move horizontally
         int oldx = people[britney].x; // save original position
         if( key[KEY_LEFT] )  people[britney].x -= PERSON_SPEED;
         if( key[KEY_RIGHT] ) people[britney].x += PERSON_SPEED;
         // collision detection
         for( int i = 0; i < numBuildings; i++ )
         {
            if( IsAdultOverlapping( people[britney], buildings[i] ) || IsOutOfBounds( people[britney] ) )
            {
               // restore original position
               people[britney].x = oldx;
               oldx = people[britney].x;
            }
         }

         // move carried tabloid
         TabloidUpdatePosition();
      }

      // find the nearest tabloid each loop (for picking up and turning on highlight)
      //SetNearestTabloid();

      if( key[KEY_SPACE] )
      {
         if( !gWasSpacePressedLastLoop )
         {
            //TabloidPickUpPutDown();
            SpacePressed();
         }
         gWasSpacePressedLastLoop = true;
		}
      else
      {
         gWasSpacePressedLastLoop = false;
      }
      
      // exit game
      if( key[KEY_ESC] )
      {
			exit( 0 );
			return 0;
		}
	} // end keypressed

   InputNPCs();

   return 1;
}

void InitBuildings()
{
   // bitmaps
   
   gBmpHouseOff      = load_bitmap( "img/b_house0.bmp", NULL );
   gBmpHouseOn       = load_bitmap( "img/b_house1.bmp", NULL );

   buildings[brithouse].bmps[0]  = load_bitmap( "img/b_brithouse0.bmp", NULL );
   buildings[brithouse].bmps[1]  = load_bitmap( "img/b_brithouse1.bmp", NULL );
   buildings[club].bmps[0]       = load_bitmap( "img/b_club0.bmp", NULL );
   buildings[club].bmps[1]       = load_bitmap( "img/b_club1.bmp", NULL );
   buildings[barber].bmps[0]     = load_bitmap( "img/b_barber0.bmp", NULL );
   buildings[barber].bmps[1]     = load_bitmap( "img/b_barber1.bmp", NULL );
   buildings[rehab].bmps[0]      = load_bitmap( "img/b_rehab0.bmp", NULL );
   buildings[rehab].bmps[1]      = load_bitmap( "img/b_rehab1.bmp", NULL );
   buildings[parishouse].bmps[0] = gBmpHouseOff;
   buildings[parishouse].bmps[1] = gBmpHouseOn;
   buildings[lohanhouse].bmps[0] = gBmpHouseOff;
   buildings[lohanhouse].bmps[1] = gBmpHouseOn;
   buildings[justinhouse].bmps[0]= gBmpHouseOff;
   buildings[justinhouse].bmps[1]= gBmpHouseOn;

   // position - upper row of lots

   buildings[parishouse].x    = 60;
   buildings[parishouse].y    = 100;
   
   buildings[brithouse].x     = 350;
   buildings[brithouse].y     = 180;
   
   buildings[club].x          = 660;
   buildings[club].y          = 100;
   
   // position - lower row of lots
   
   buildings[barber].x        = 60;
   buildings[barber].y        = 450;
   
   buildings[lohanhouse].x    = 260;
   buildings[lohanhouse].y    = 450;
   
   buildings[rehab].x         = 460;
   buildings[rehab].y         = 450;
   
   buildings[justinhouse].x   = 660;
   buildings[justinhouse].y   = 450;

   // size
   for( int i = 0; i < numBuildings; i++ )
   {
      buildings[i].bmp = buildings[i].bmps[0];
      buildings[i].w = buildings[i].bmp->w;
      buildings[i].h = buildings[i].bmp->h;
      buildings[i].half_w = buildings[i].w / 2;
      buildings[i].half_h = buildings[i].h / 2;
      //buildings[i].containedPerson = NULL;
      buildings[i].prevPersonX = 0;
      buildings[i].prevPersonY = 0;
      buildings[i].statPopularityDelta = 0;
      buildings[i].statHealthDelta = 0;
      buildings[i].statFunDelta = 0;
   }

   // custom stat deltas
   buildings[club].statPopularityDelta    = STAT_CLUB_POPULARITY;
   buildings[club].statHealthDelta        = STAT_CLUB_HEALTH;
   buildings[club].statFunDelta           = STAT_CLUB_FUN;
   buildings[barber].statPopularityDelta  = STAT_BARBER_POPULARITY;
   buildings[barber].statHealthDelta      = STAT_BARBER_HEALTH;
   buildings[barber].statFunDelta         = STAT_BARBER_FUN;
   buildings[rehab].statPopularityDelta   = STAT_REHAB_POPULARITY;
   buildings[rehab].statHealthDelta       = STAT_REHAB_HEALTH;
   buildings[rehab].statFunDelta          = STAT_REHAB_FUN;
}

void InitPeople()
{
   // bitmaps
   people[britney].bmp  = load_bitmap( "img/britney3.bmp", NULL );
   people[paris].bmp    = load_bitmap( "img/paris.bmp", NULL );

   // position
   /*
   people[britney].x    = 20;
   people[britney].y    = 20;
   people[paris].x      = 20;
   people[paris].y      = 20;
   */

   // size
   for( int i = 0; i < numPeople; i++ )
   {
      people[i].w = people[i].bmp->w;
      people[i].h = people[i].bmp->h;
      people[i].half_w = people[i].w / 2;
      people[i].half_h = people[i].h / 2;
      people[i].carryState = carry_none;
      people[i].buildingCurrent = NULL;
      people[i].buildingHome = &(buildings[i]);
      people[i].loopsTilNextAction = Random( NPC_ACTION_LOOPS_MIN, NPC_ACTION_LOOPS_MAX );
      people[i].state = personstate_default;
      people[i].stateTabloid = NULL;
      people[i].stateBuilding = NULL;
      people[i].x = people[i].buildingHome->x - people[i].buildingHome->half_w;
      people[i].y = people[i].buildingHome->y - people[i].buildingHome->half_h;
   }

   // test
   //people[paris].buildingHome = &(buildings[parishouse]);
}

void InitTabloids()
{
   gBmpTabloidHighlightOff  = load_bitmap( "img/tabloid0.bmp", NULL );
   gBmpTabloidHighlightOn   = load_bitmap( "img/tabloid1.bmp", NULL );

   // size
   for( int i = 0; i < numTabloids; i++ )
   {
      tabloids[i].bmp = gBmpTabloidHighlightOff;
      tabloids[i].w = tabloids[i].bmp->w;
      tabloids[i].h = tabloids[i].bmp->h;
      tabloids[i].half_w = tabloids[i].w / 2;
      tabloids[i].half_h = tabloids[i].h / 2;
      tabloids[i].x = Random( 0, SCREEN_W );
      tabloids[i].y = Random( 0, SCREEN_H );
      tabloids[i].fireState = fire_off;
      tabloids[i].loopsUntilBurnedOut = 0;
   }
}

void InitFirePatch()
{
   gFirePatch.x = 250;
   gFirePatch.y = 50;
   gFirePatch.numFrames = 2;
   gFirePatch.loopsPerFrame = LOOPS_PER_FRAME;
   FirePatchSetVisible( false );
   for( int i = 0; i < MAX_FRAMES; i++ )
   {
      gFirePatch.frames[i] = NULL;
   }
   gFirePatch.frames[0] = load_bitmap( "img/firepatch0.bmp", NULL );
   gFirePatch.frames[1] = load_bitmap( "img/firepatch1.bmp", NULL );
   gFirePatch.bmp = gFirePatch.frames[0];
   gFirePatch.w = gFirePatch.bmp->w;
   gFirePatch.h = gFirePatch.bmp->h;
   gFirePatch.half_w = gFirePatch.w / 2;
   gFirePatch.half_h = gFirePatch.h / 2;
}

void InitGlobals()
{
   gBackBuf = create_bitmap( SCREEN_WIDTH, SCREEN_HEIGHT );
   gWasSpacePressedLastLoop = false;
   gNearestTabloid = NULL;
   gNearestBuilding = NULL;
   
   gStatCurLoop = 0;
   gPopularity = STAT_INIT_POPULARITY;
   gHealth = STAT_INIT_HEALTH;
   gFun = STAT_INIT_FUN;
   
   gLoopsUntilDelivery = 0;

   gBmpFire = load_bitmap( "img/fire.bmp", NULL );

   InitFirePatch();
}

void Init()
{
   InitGlobals();
   //InitPeople();
   InitBuildings();
   InitPeople();
   InitTabloids();
}

int main( void )
{
   // INIT

   // always do this for allegro programs
   if (allegro_init() != 0)
      return 1;

   install_keyboard(); 
   install_mouse();

   // graphics setup
   set_color_depth( 16 );
   if ( set_gfx_mode( GFX_AUTODETECT_WINDOWED, SCREEN_WIDTH, SCREEN_HEIGHT, 0, 0 ) != 0 )
	{
      set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
      allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
      return 1;
   }   

   Init();
   
   // GAME LOOP

   while( 1 )
   {
      if( !Input() )
      {
         break;
      }
      
      Update();
      Draw();

      yield_timeslice();
   
   } // end while

   return 0;
}

END_OF_MAIN()
